<?php

declare(strict_types=1);

class CapabilityPlaybackController
{
    const capabilityPrefix = 'PlaybackController';
    const DATE_TIME_FORMAT = 'o-m-d\TH:i:s\Z';

    use HelperPlaybackDevice;

    public static function computeProperties($configuration)
    {
    	return [];
    }

    public static function getColumns()
    {
        return [
            [
                'label' => 'Playstate Variable',
                'name'  => self::capabilityPrefix . 'ID',
                'width' => '250px',
                'add'   => 0,
                'edit'  => [
                    'type' => 'SelectVariable'
                ]
            ]
        ];
    }

    public static function getStatus($configuration)
    {
        return self::getPlaybackCompatibility($configuration[self::capabilityPrefix . 'ID']);
    }

    public static function getStatusPrefix()
    {
        return 'PlayState: ';
    }

    public static function doDirective($configuration, $directive, $payload, $emulateStatus)
    {
        $setPlayState = function ($configuration, $value, $emulateStatus)
        {
            if ($check=self::setPlaystate($configuration[self::capabilityPrefix . 'ID'], $value)) {
                $properties = [];
                if ($emulateStatus) {
                    $properties = self::computeProperties($configuration);
                } else {
                    $i = 0;
                    while (($value != self::getPlaystate($configuration[self::capabilityPrefix . 'ID'])) && $i < 10) {
                        $i++;
                        usleep(100000);
                    }
                    $properties = self::computeProperties($configuration);
                }
                return [
                    'properties'     => $properties,
                    'payload'        => new stdClass(),
                    'eventName'      => 'Response',
                    'eventNamespace' => 'Alexa'
                ];
            } else {
            	if(is_null($check)){
            		$payload =  [
                        'type' 		=> 'NO_SUCH_ENDPOINT'
                    ];
            	} else {
            		$payload =  [
                        'type' 		=> 'INVALID_VALUE',
                    	'message'	=> "Playmode $value not supported"
            		];
            	}
                return [
                    'payload'        => $payload,
                    'eventName'      => 'ErrorResponse',
                    'eventNamespace' => 'Alexa'
                ];
            }
        };
        
        switch ($directive) {
            case 'ReportState':
                return [
                    'properties'     => self::computeProperties($configuration),
                    'payload'        => new stdClass(),
                    'eventName'      => 'StateReport',
                    'eventNamespace' => 'Alexa'
                ];
                break;
            case 'FastForward': return $setPlayState($configuration,1,$emulateStatus);
            case 'Next': return $setPlayState($configuration,2,$emulateStatus);
            case 'Pause': return $setPlayState($configuration,3,$emulateStatus);
            case 'Play': return $setPlayState($configuration,4,$emulateStatus);
        	case 'Previous': return $setPlayState($configuration,5,$emulateStatus);	
        	case 'Rewind': return $setPlayState($configuration,6,$emulateStatus);
       		case 'StartOver': return $setPlayState($configuration,7,$emulateStatus);
        	case 'Stop': return $setPlayState($configuration,8,$emulateStatus);
//             case 'ChangeChannel':
//             	$newValue = $payload['channel']['number'];
//             	return $switchChannel($configuration, $newValue, $emulateStatus);
//             case 'SkipChannels':;	
//             	$step = $payload['channelCount'];
//             	return $skipChannels($configuration, $step, $emulateStatus);
            default:
                throw new Exception('Command is not supported by this trait!');
        }
    }

    public static function getObjectIDs($configuration)
    {
        return [
            $configuration[self::capabilityPrefix . 'ID']
        ];
    }

    public static function supportedDirectives()
    {
        return [
            'ReportState',
            'FastForward',
            'Next',
            'Pause',
            'Play',
        	'Previous',	
        	'Rewind',	
       		'StartOver',
        	'Stop',
        ];
    }

    public static function supportedCapabilities()
    {
        return [
            'Alexa.PlaybackController'
        ];
    }

    public static function supportedProperties($realCapability, $configuration)
    {
        return [];
    }

   	public static function getCapabilityInformation($configuration)
    {
        $capabilitiesInfo = [];
        $capabilities = self::supportedCapabilities();
        foreach ($capabilities as $realCapability) {
            $capabilitiesInfo[] = [
                    'type'       => 'AlexaInterface',
                    'interface'  => $realCapability,
                    'version'    => '3',
                    'properties' => [
                        'proactivelyReported' => false,
                        'retrievable'         => true
                    ]
            ];
        }
        return $capabilitiesInfo;
    }
    

}
