<?php

declare(strict_types=1);

trait HelperChannelDevice
{
	private static $number_defs = [
    			'de'=> ['null','eins','zwei','drei','vier','fünf','sechs','sieben','acht','neun','zehn','elf','zwölf','dreizehn','vierzehn','fünfzehn','sechzehn'],
    			'en'=> ['null', 'one', 'two', 'three', 'four', 'five', 'six', 'seven', 'eight', 'nine', 'ten','eleven', 'twelve', 'thirteen', 'fourteen', 'fifteen', 'sixteen']
    ];
    private static function getChannelCompatibility($variableID)
    {
    	
        if (!IPS_VariableExists($variableID)) {
            return 'Missing';
        }

        $targetVariable = IPS_GetVariable($variableID);

        if ($targetVariable['VariableType'] != 1 /* Integer */ ) {
            return 'Int required';
        }

        if ($targetVariable['VariableCustomProfile'] != '') {
            $profileName = $targetVariable['VariableCustomProfile'];
        } else {
            $profileName = $targetVariable['VariableProfile'];
        }

        if (!IPS_VariableProfileExists($profileName)) {
            return 'Profile required';
        }

        if ($targetVariable['VariableCustomAction'] != '') {
            $profileAction = $targetVariable['VariableCustomAction'];
        } else {
            $profileAction = $targetVariable['VariableAction'];
        }

        if (!($profileAction > 10000)) {
            return 'Action required';
        }

        return 'OK';
    }
   
    private static function convertNumbers($value){
    	if(is_numeric($value)){
    		return $value;
    	}
    	foreach (self::$number_defs as $numbers){
	    	if($number=array_search(strtolower($value), $numbers)){
	    		return $number;
	    	}
	    	foreach ($numbers as $number=>$name){
	    		if(stripos($value.' ',$name.' ')){
	    			return trim(str_ireplace($name.' ',' '.$number,$value.' '));
	    		}
	    	}
	    }
   		return $value;
    }
    
    private static function getChannelNumber($variableID)
    {
        if (!IPS_VariableExists($variableID)) {
            return 0;
        }
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] != 1 /* Integer */) {
            return 0;
        }
        if ($targetVariable['VariableCustomProfile'] != '') {
             $profileName = $targetVariable['VariableCustomProfile'];
         } else {
             $profileName = $targetVariable['VariableProfile'];
        }
        if(!IPS_VariableProfileExists($profileName)){
         	return 0;
        }
        if(empty(GetValueFormatted($variableID))){
        	return 0;
        }
        return GetValue($variableID);
    }
    private static function skipChannels($variableID, $step, &$newChannel){
        if (!IPS_VariableExists($variableID)) {
            return false;
        }
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] != 1 /* Integer */) {
            return false;
        }
        if ($targetVariable['VariableCustomProfile'] != '') {
             $profileName = $targetVariable['VariableCustomProfile'];
         } else {
             $profileName = $targetVariable['VariableProfile'];
        }
        if(!IPS_VariableProfileExists($profileName)){
         	return false;
        }
        $profile = IPS_GetVariableProfile($profileName);
    	$value = GetValue($variableID);
    	$temp = array_filter($profile['Associations'],function($a)use($value){
    		return $a['Value']>=$value;
    	});
   		$index = (int)key($temp);
   		$count   = count($profile['Associations']);
   		if($step<0){
   			 $index-=$step;
   			 if($index < 0){
   				$index = 0;
   			}
   		}
   		else 
   		{
   			$index+=$step;
   			if($index>=$count){
   				$index=$count-1;
   			}
   		}
   		if (empty($profile['Associations'][$index])){
   			return false;
   		}
    	return self::setChannel($variableID, $newChannel=$profile['Associations'][$index]['Value']);
    }

    private static function setChannel($variableID, &$value)
    {
        if (!IPS_VariableExists($variableID)) {
            return null;
        }
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] != 1 /* Integer */) {
            return null;
        }
        if ($targetVariable['VariableCustomAction'] != 0) {
            $profileAction = $targetVariable['VariableCustomAction'];
        } else {
            $profileAction = $targetVariable['VariableAction'];
        }
        if ($profileAction < 10000) {
            return null;
        }
        if ($targetVariable['VariableCustomProfile'] != '') {
            $profileName = $targetVariable['VariableCustomProfile'];
        } else {
            $profileName = $targetVariable['VariableProfile'];
        }
       	
       	
       	if (!IPS_VariableProfileExists($profileName)) {
            return null;
        }
        
        $profile = IPS_GetVariableProfile($profileName);
		$found = false;
		$value = self::convertNumbers($value);
        if(is_numeric($value)){
        	foreach ($profile['Associations'] as $a){
        		if($a['Value']==$value){
					$found=$a['Name'];
					break;
        		}
        	}
        }
        else {
        	foreach ($profile['Associations'] as $a){
        		if(strcasecmp($a['Name'],$value)==0){
					$found=$a['Name'];
					$value=$a['Value'];
					break;
        		}
        	}
        }
        if(!$found){
        	return false;
        }
		
	    if (IPS_InstanceExists($profileAction)) {
            IPS_RunScriptText('IPS_RequestAction(' . var_export($profileAction, true) . ', ' . var_export(IPS_GetObject($variableID)['ObjectIdent'], true) . ', ' . var_export($value, true) . ');');
        } elseif (IPS_ScriptExists($profileAction)) {
            IPS_RunScriptEx($profileAction, ['VARIABLE' => $variableID, 'VALUE' => $value, 'NAME'=>$found, 'SENDER' => 'ChannelControl']);
        } else {
            return false;
        }
        return true;
    	
    }
    
    private static function getChannelValue($variableID)
    {
    	if($number = self::getChannelNumber($variableID)){
    		return [
    			'number'	=> "$number",
    			'callSign' 	=> GetValueFormatted($variableID)
    		];
    	}
        return [
        	'number'	=> "$number"
        ];
    }

}
