<?php

declare(strict_types=1);

trait HelperInputDevice
{
// 	private static const allowed_inputs = [
// 		'AUX 1','AUX 2','AUX 3','AUX 4','AUX 5','AUX 6','AUX 7','BLURAY','CABLE','CD','COAX 1','COAX 2','COMPOSITE 1','DVD','GAME','HD RADIO','HDMI 1','HDMI 2','HDMI 3','HDMI 4','HDMI 5','HDMI 6','HDMI 7','HDMI 8','HDMI 9','HDMI 10','HDMI ARC','INPUT 1','INPUT 2','INPUT 3','INPUT 4','INPUT 5','INPUT 6','INPUT 7','INPUT 8','INPUT 9','INPUT 10','IPOD','LINE 1','LINE 2','LINE 3','LINE 4','LINE 5','LINE 6','LINE 7','MEDIA PLAYER','OPTICAL 1','OPTICAL 2','PHONO','PLAYSTATION','PLAYSTATION 3','PLAYSTATION 4','SATELLITE','SMARTCAST','TUNER','TV','USB DAC','VIDEO 1','VIDEO 2','VIDEO 3','XBOX'
// 	];
	
    private static function getInputCompatibility($variableID)
    {
        if (!IPS_VariableExists($variableID)) {
            return 'Missing';
        }
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] != 1 /* Integer */ ) {
            return 'Int required';
        }
        if ($targetVariable['VariableCustomProfile'] != '') {
            $profileName = $targetVariable['VariableCustomProfile'];
        } else {
            $profileName = $targetVariable['VariableProfile'];
        }
        if (!IPS_VariableProfileExists($profileName)) {
            return 'Profile required';
        }        
        if ($targetVariable['VariableCustomAction'] != '') {
            $profileAction = $targetVariable['VariableCustomAction'];
        } else {
            $profileAction = $targetVariable['VariableAction'];
        }
        if (!($profileAction > 10000)) {
            return 'Action required';
        }
       	$profile = IPS_GetVariableProfile($profileName);
        $keys = array_map(function($a){return $a['Value'];},$profile['Associations']);
        $min = min($keys);
        $max = max($keys);
        if($min<0 || $max > 60){
        	return "Profile min/max id mismatch (allowed 0-60)";
        }
        return 'OK';
    }
	
    private static function getInputValue($variableID)
    {
        if (!IPS_VariableExists($variableID)) {
			return '';
        }
        $value = '';
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] == 1){
			$value = GetValueFormatted($variableID);
        }
 		return $value;
    }
    private static function setInputValue($variableID, $value)
    {
		if (!IPS_VariableExists($variableID)) {
			return null;
       	}
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] != 1 /* Integer */ ) {
            return null;
        }
        if ($targetVariable['VariableCustomProfile'] != '') {
            $profileName = $targetVariable['VariableCustomProfile'];
        } else {
            $profileName = $targetVariable['VariableProfile'];
        }
        if (!IPS_VariableProfileExists($profileName)) {
            return null;
        }        
        if ($targetVariable['VariableCustomAction'] != '') {
            $profileAction = $targetVariable['VariableCustomAction'];
        } else {
            $profileAction = $targetVariable['VariableAction'];
        }
        if (!($profileAction > 10000)) {
            return null;
        }
       	$profile = IPS_GetVariableProfile($profileName);
       	$found=false;
       	foreach($profile['Associations'] as $a){
       		if($found=strcasecmp($a['Name'],$value)==0){
       			$name = $value;
       			$value = $a['Value'];
       			break;
       		}
       	}
       	if(!$found){
       		return false;
       	}
       	 
       	if (IPS_InstanceExists($profileAction)) {
            IPS_RunScriptText('IPS_RequestAction(' . var_export($profileAction, true) . ', ' . var_export(IPS_GetObject($variableID)['ObjectIdent'], true) . ', ' . var_export($value, true) . ');');
        } elseif (IPS_ScriptExists($profileAction)) {
            IPS_RunScriptEx($profileAction, ['VARIABLE' => $variableID, 'VALUE' => $value, 'NAME'=>$name, 'SENDER' => 'InputControl']);
        } else {
            return false;
        }
        return true;
    }
    
}
