<?php

declare(strict_types=1);

trait HelperPlaybackDevice
{
	
	private static function getPlaybackCompatibility($variableID)
    {
        if (!IPS_VariableExists($variableID)) {
            return 'Missing';
        }

        $targetVariable = IPS_GetVariable($variableID);

        if ($targetVariable['VariableType'] != 1 /* Integer */ ) {
            return 'Int required';
        }

        if ($targetVariable['VariableCustomProfile'] != '') {
            $profileName = $targetVariable['VariableCustomProfile'];
        } else {
            $profileName = $targetVariable['VariableProfile'];
        }

        if (!IPS_VariableProfileExists($profileName)) {
            return 'Profile required';
        }

        $profile = IPS_GetVariableProfile($profileName);
        $keys = array_map(function($a){return $a['Value'];},$profile['Associations']);
        $min = min($keys);
        $max = max($keys);
        if($min<0 || $max > 8){
        	return "Profile min/max id mismatch (allowed 1-8)";
        }

        if ($targetVariable['VariableCustomAction'] != '') {
            $profileAction = $targetVariable['VariableCustomAction'];
        } else {
            $profileAction = $targetVariable['VariableAction'];
        }

        if (!($profileAction > 10000)) {
            return 'Action required';
        }

        return 'OK';
    }
	
   
	private static function getPlaystate($variableID) {
        if (!IPS_VariableExists($variableID)) {
            return 0;
        }
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] != 1 /* Integer */ ) {
            return 0;
        }
		$value = $targetVariable['VariableValue'];
        
        return $value > 0 && $value < 9 ? $value : 0;
	}

	private static function setPlaystate($variableID, $value) {
        if (!IPS_VariableExists($variableID)) {
            return null;
        }
        $targetVariable = IPS_GetVariable($variableID);
        if ($targetVariable['VariableType'] != 1 /* Integer */) {
            return null;
        }
        if ($targetVariable['VariableCustomProfile'] != '') {
            $profileName = $targetVariable['VariableCustomProfile'];
        } else {
            $profileName = $targetVariable['VariableProfile'];
        }
       	if (!IPS_VariableProfileExists($profileName)) {
            return null;
        }
        if ($targetVariable['VariableCustomAction'] != 0) {
            $profileAction = $targetVariable['VariableCustomAction'];
        } else {
            $profileAction = $targetVariable['VariableAction'];
        }
        if ($profileAction < 10000) {
            return null;
        }
        $profile = IPS_GetVariableProfile($profileName);
	    $found = false;
	    foreach ($profile['Associations'] as $a){
       		if($found=$a['Value']==$value){
       			$name = $a['Name'];
				break;
        	}
        }
        if(!$found){
        	return false;
        }
        if (IPS_InstanceExists($profileAction)) {
            IPS_RunScriptText('IPS_RequestAction(' . var_export($profileAction, true) . ', ' . var_export(IPS_GetObject($variableID)['ObjectIdent'], true) . ', ' . var_export($value, true) . ');');
        } elseif (IPS_ScriptExists($profileAction)) {
            IPS_RunScriptEx($profileAction, ['VARIABLE' => $variableID, 'VALUE' => $value, 'NAME'=>$name, 'SENDER' => 'PlayControl']);
        } else {
            return null;
        }
        return true;
	}
	
	
}
